//====================================================================
// BATTLE-RELATED MODIFICATIONS
//====================================================================


//-----------------------------------------------------------------------------
// GRAPHICS MODIFICATIONS
//-----------------------------------------------------------------------------
// Insert new "MISS" battle sprite
org $CFC60; incbin bin/graphics-battle-miss.bin

// remove the little "MP" icon from the battle windows
org $2a1de; fill $14,$EA



//-----------------------------------------------------------------------------
// SHORT SPELL NAME MODIFICATIONS
//-----------------------------------------------------------------------------

org $28a0c0; db "****************NEW TEXT: SHORT SPELL NAMES"
org $28a2a0; db "****************NEW TEXT: ALT SHORT SPELL NAMES"
org $2a0ff; jsl loadshortspellname_battle1
org $2a117; jsl loadshortspellname_battle2




//-----------------------------------------------------------------------------
// LONG SPELL NAME-RELATED MODIFICATIONS
//-----------------------------------------------------------------------------
// allow for expanded spell names in battle victory text
// new max length is now technically 0x20 bytes but that's crazy of course
// names can actually only be 14 letters long to fit within the text window, 15 if you count the initial magic symbol
org $2e8900; db "****************NEW TEXT: EXPANDED SPELL NAMES"
org $289780; db "****************NEW TEXT: ALT EXPANDED SPELL NAMES"

org $2a57b; db $20
org $2a573; jsl expandspellnamesvictory1
org $2a57e; jsl expandspellnamesvictory2
org $2a56b; jsl expandspellnamesvictory3; nop; nop;




//-----------------------------------------------------------------------------
// TEXT SYSTEM MODIFICATIONS
//-----------------------------------------------------------------------------

// redirect game to load battle dialogue from new expanded location
org $2E9A00; db "****************NEW TEXT: EXPANDED BATTLE DIALOGUE POINTERS"
org $2E9BC0; db "****************NEW TEXT: EXPANDED BATTLE DIALOGUE STRINGS"
org $2c90a; db $40,$9a,$2e
org $2c910; db $41,$9a,$2e
org $2c916; db $2e

// redirect game to load battle notices from new expanded location
org $2E94A0; db "****************NEW TEXT: EXPANDED BATTLE NOTICE POINTERS"
org $2E9560; db "****************NEW TEXT: EXPANDED BATTLE NOTICE STRINGS"
org $2cc08; db $e0,$94,$2e
org $2cc11; db $2e

// Expand battle message/notice window to the maximum length, it looks ugly
// but I feel the benefit of longer text strings outweighs this negative
org $292e0; db $01   // move battle msg window all the way left
org $292e6; db $1e   // extend battle msg window all the way across
org $292f9; db $2a   // move battle msg text left

// allow item icons to appear in "___ stolen!" text when appropriate
// also allows expanded item names, but window space limits means some
// long names will get cut short
org $2a5a5; lda $2b8030,x
org $2a598
jsl expanditems_casting
jsl expand_stolen_item_names
nop
lda #$10


//-----------------------------------------------------------------------------
// CURRENT ACTION (AKA CASTING) WINDOW-RELATED MODIFICATIONS
//-----------------------------------------------------------------------------
// My goal here is to allow for longer text in the "current action" window
// that previously before. Not only that, I want the window to snap to the
// current string's size to look more professional and cool.

//-----------------------------------------------------------------------------
// expand the standard casting window
org $29369
jsl strlen
sta $0754  // store length of the expanded string, this address should be safe?
jsl autoformat_casting_window_pt1
nop


//-----------------------------------------------------------------------------
// expand the back attack casting window
org $2930d
jsl strlen
sta $0754
jsl autoformat_casting_window_pt1_backattack
nop


//-----------------------------------------------------------------------------
// allow for extended spell names in "now casting" battle text box
org $2cbd0; jsl expandspellnamescasting;
org $2cbdd; jsl autoformat_casting_window_pt3; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop


//-----------------------------------------------------------------------------
// allow for extended enemy action names in "now casting" battle text box
org $2ba030; db "****************NEW TEXT: EXPANDED ENEMY BATTLE ACTION STRINGS"
org $288000; db "****************NEW TEXT: ALTERNATE BATTLE ACTION STRINGS"

org $2cbab; jsl expandactionnames; nop
org $29382; jsl autoformat_casting_window_pt2; nop; nop;
org $2cbb2; jsl autoformat_casting_window_pt4; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop


//-----------------------------------------------------------------------------
// allow for extended item names in the casting window
org $2cb77; fill $1B,$EA
org $2cb77
jsl expanditems_casting
tdc
tay
ldx $2a
jsl autoformat_casting_window_pt5
tdc


//-----------------------------------------------------------------------------
// allow for differently sized action text (Steal, Cover, etc.) in casting window
// we're not really expanding the size of this text, just making the window snap better
org $2cb4a; db $07
org $2cb54; fill $0E,$EA
org $2cb54; jsl autoformat_casting_window_pt6




//-----------------------------------------------------------------------------
// GRUMPY BATTLE TEXT MODIFICATIONS
//-----------------------------------------------------------------------------
// A few strings in battle use the 4bpp background, so they don't have access
// to the full alphabet tileset like ordinary battle text. This means they're
// a real hassle to work with, so I call them "grumpy".
//
// The whole situation gets kinda weird and we need to define which handful of
// letters we WILL be using ahead of time.

// Modify what letters we'll be using ahead of time
org $16fb87
db $4E,$51,$C7,$44,$6A,$6E,$6F,$45
db $60,$61,$69,$5F,$53,$72,$C1,$8C
db $90,$7F,$80,$81,$82,$83,$84,$85
db $86,$87,$88,$89,$16,$17,$18,$FA
db $FB,$FC,$FD,$FE,$FF

// Make it print "MP Cost" in battle magic menu
org $2975D; db $DB,$DC,$FF,$DE,$DF,$E0,$E1

// Make it say "Def."
org $29ab7; db $FF,$FF,$FF,$FF,$E2,$E3,$E4,$E9

// Make it say "Rows"
org $29abf; db $FF,$FF,$FF,$FF,$E7,$DF,$E8,$E0




//-----------------------------------------------------------------------------
// STATUS AILMENT TEXT MODIFICATIONS (affects main menus somewhat too)
//-----------------------------------------------------------------------------
// Move and expand status ailment text, also affects the text in the main
// menus outside of battle, but in most cases we'll never see those instances
// because we're removing class names from most of those. But "KOed" will
// definitely appear in main menus

org $2E9300; db "****************NEW TEXT: LONG STATUS AILMENT POINTERS"
org $2E9380; db "****************NEW TEXT: LONG STATUS AILMENT STRINGS"

org $2a345; dw $000A        // new max length of status strings
org $2a32a; lda $2e9340,x   // new location of status string pointers
org $2a330; lda $2e9341,x
org $2a336; lda #$2e        // new bank of status string text

org $2a51c; lda $2e9340,x
org $2a522; lda $2e9341,x
org $2a528; lda #$2e




//-----------------------------------------------------------------------------
// ITEM HANDEDNESS STRING HANDLING MODIFICATIONS
//-----------------------------------------------------------------------------
// Expands the "handedness" strings in the battle item menu.
// Originally, these strings are generated on the fly and are constantly
// switching depending on the active character. I tried to expand these strings
// but eventually realized there probably wasn't enough free VRAM, so I decided
// to bake in "Right Hand" and "Left Hand" ahead of time and then disable the
// on-the-fly stuff, which isn't very helpful to begin with. Even in FF2us,
// the programmers just leave the strings for the dominant hand empty and stick
// to the short string limitations. Poo to that!

org $2BF2A0; db "****************NEW TEXT: ITEM HANDEDNESS IN BATTLE"

// NOP all the original code for on-the-fly string changes
org $297c6; fill $25,$EA

// Moving the battle window creation code elsewhere for easier expansion and editing
org $29b1c; fill $0D,$EA
org $29b1c; jml battle_handedness_jump_off_point

//-----------------------------------------------------------------------------
// Most of this is the original battle window creation code, except for the JSL
// to our custom string code. Note I had to keep this within the original bank,
// so I wrote this over a bunch of FFs at the end of the bank. I dunno if those
// FFs are actually used... hopefully not.

org $2ffd0
battle_handedness_jump_off_point:
-
jsr $9b59
pha
jsr $9bc7
pla
jsl write_battle_handedness_strings
inc
cmp #$07
bne -
jml $29b27




//-----------------------------------------------------------------------------
// BATTLE MENU-RELATED MODIFICATIONS
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// expand battle command menus and command text lengths from 5 letters to 7

org $2e8120; db "****************NEW TEXT: EXPANDED BATTLE MENU STRINGS"

org $29d16; db $07  // max length of new command strings

org $29d43; db $09
org $29d5b; db $09
org $29cd7; db $07
org $29ce1; db $1C

org $29d39;  lda $2E8160,x
org $16fe66; db $04                  // starting x position of window
org $16fe68; db $09                  // width of window
org $16fe58; db $f0                  // move command text left
org $2b991;  db $18                  // move cursor left 16 pixels, normally is $28
org $16fc51; db $97,$a3,$af,$bb,$c7  // move cursor y-positions up slightly for nicer alignment

// because we're expanding stuff, we need to find a blank area of RAM with more room
// originally they start at 7e8cb2 and have 0x64 (100 dec) bytes between them
// our new versions require 0x8c size blocks (140 dec) instead, so we'll need to move them all to hopefully free space
// *******WARNING + NOTICE!!!*******
// I've chosen the $7ef600 to $7ef8bb area for my temporary storage
// This area seems unused during battles but could cause trouble sometime
org $16fe54; db $0e,$0a,$00,$f6               // expanding length of windows + text and repointing where they should be unpacked in RAM
org $16fead; dw $f600,$f68c,$f718,$f7a4,$f830 // also repointing where in RAM to store/read the unpacked text
org $299a0;  db $8c                           // let the game know the size between each set of 5 commands is bigger now


//-----------------------------------------------------------------------------
// adjust cursor positions in battle item menu to look better
org $16fc59; db $08,$78               // cursor x positions
org $16fc5b; db $9b,$a7,$b3,$bf,$cb   // cursor y positions


//-----------------------------------------------------------------------------
// remove numbers from enemy names + expand enemy names
org $2BB7B0; db "****************NEW TEXT: EXPANDED ENEMY NAMES"
org $2BD3E0; db "****************ALT ENEMY NAMES"

org $2a7d9; jsl expandenemynames1
org $2a7e1; db $0A

org $2a7e4; jml expandenemynames2


//-----------------------------------------------------------------------------
// reconstruct the enemy name window to allow names to fill the window and skip the numbers
org $2a349; cpy #$000A
org $2a7bf; ldx #$000A
org $2a3c0; db $03
org $2a3d7; db $03
org $16ff63
db $0C,$00,$01
db $0C,$01,$01
db $0C,$02,$01
db $0C,$03,$01
db $42,$42,$42,$42,$42,$42,$42,$42,$42,$42,$42,$42




//====================================================================
// NEW CUSTOM CODE GOES HERE
//====================================================================

org $2BF500
db "****************NEW CODE: BATTLE TEXT IMPROVEMENTS"

//-----------------------------------------------------------------------------
expanditems_casting:
stz $2a
stz $2b
lda $26
asl; rol $2b // id * 0x02
asl; rol $2b // id * 0x04
asl; rol $2b // id * 0x08
asl; rol $2b // id * 0x10
asl; rol $2b // id * 0x20
sta $2a
rtl


//-----------------------------------------------------------------------------
// we're moving the standard short spell names to the expanded area of the rom
// so let's let the battle system know
loadshortspellname_battle1:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $28a2d0,x   // if we're here, we're using tsushiy's custom enemy patch, so use custom strings
rtl

+
lda $28a0f0,x   // if we're here, we're using the original patch, so use original strings
rtl


//-----------------------------------------------------------------------------
loadshortspellname_battle2:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $28a2d1,x   // if we're here, we're using tsushiy's custom enemy patch, so use custom strings
rtl

+
lda $28a0f1,x   // if we're here, we're using the original patch, so use original strings
rtl


//-----------------------------------------------------------------------------
expandspellnamesvictory1:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $2897c0,x   // if we're here, we're using tsushiy's custom enemy patch, so use custom strings
rtl

+
lda $2e8930,x   // if we're here, we're using the original patch, so use original strings
rtl


//-----------------------------------------------------------------------------

expandspellnamesvictory2:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $2897c1,x   // if we're here, we're using tsushiy's custom enemy patch, so use custom strings
rtl

+
lda $02e8931,x
rtl


//-----------------------------------------------------------------------------
// new custom code for expanded spell names in battle victory text
expandspellnamesvictory3:
stz $2b
lda $26
asl; rol $2b // id * 0x02
asl; rol $2b // id * 0x04
asl; rol $2b // id * 0x08
asl; rol $2b // id * 0x10
asl; rol $2b // id * 0x20
sta $2a
ldy #$0000
rtl


//-----------------------------------------------------------------------------
expandspellnamescasting:
stz $2b
lda $26
asl; rol $2b // id * 0x02
asl; rol $2b // id * 0x04
asl; rol $2b // id * 0x08
asl; rol $2b // id * 0x10
asl; rol $2b // id * 0x20
sta $2a
tdc
rtl


//-----------------------------------------------------------------------------
expandenemynames1:
asl
asl
asl
asl
asl
tax
rtl


//-----------------------------------------------------------------------------
expandenemynames2:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $2bd400,x   // if we're here, tsushiy's enemy patch has been applied, so let's use the alternate enemy names
jml $2a7e8

+
lda $2BB7E0,x   // if we're here, we're using the original enemy names
jml $2a7e8


//-----------------------------------------------------------------------------
expandactionnames:
stz $2b
lda $26
asl; rol $2b // id * 0x02
asl; rol $2b // id * 0x04
asl; rol $2b // id * 0x08
asl; rol $2b // id * 0x10
asl; rol $2b // id * 0x20
sta $2a
ldx $2a
rtl


//-----------------------------------------------------------------------------
autoformat_casting_window_pt1:
ldx $00
phx

// this part adjusts the window's starting x position
// we're basically doing Xpos = (19 - stringlength) + 0xA
lda #$13
sec
sbc $0754
clc
adc #$0A
ldx #$0000
tax
stx $ef56

// now we'll adjust the window's width
// we're basically doing Width = stringlength + 2
lda #$04    // the game expects this 04 to be there, it's of no concern to us though
xba
lda $0754   // get the string's length
clc
adc #$02
tax
stx $ef58

lda #$00
xba
lda #$00

plx
stx $00
rtl


//-----------------------------------------------------------------------------

autoformat_casting_window_pt1_backattack:
ldx $00
phx

// in back attack battle, the window's starting x pos will always be the same

// this part adjusts the window's starting x position
// we're basically doing Xpos = (19 - stringlength) + 0xA
ldx #$0002
stx $ef56

// now we'll adjust the window's width
// we're basically doing Width = stringlength + 2
lda #$04    // the game expects this 04 to be there, it's of no concern to us though
xba
lda $0754   // get the string's length
clc
adc #$02
tax
stx $ef58

lda #$00
xba
lda #$00

plx
stx $00
rtl


//-----------------------------------------------------------------------------
autoformat_casting_window_pt2:

lda $0754  // get length of current expanded string
asl        // multiply length by 2
ldx $00
phx
ldx #$db62
stx $00
sta $00
lda #$62
sec
sbc $00
sta $00
ldx $00
stx $ef52
plx
stx $00
rtl


//-----------------------------------------------------------------------------
// this function copies only the needed number of chars so that the casting window looks right
autoformat_casting_window_pt3:

acw3_loop:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $2897c0,x   // if we're here, we're using tsushiy's patch, so use alternate action strings
jmp acw3_store

+
lda $2e8930,x   // if we're here, we're using the original patch, so use original strings

acw3_store:
sta $74fd,y
cmp #$ff
beq acw3_leave
iny
inx
cpy #$0014
bne acw3_loop

acw3_leave:
jsl strlen
dec
sta $0754
rtl


//-----------------------------------------------------------------------------
// this function copies only the needed number of chars so that the casting window looks right
// this one handles enemy actions
autoformat_casting_window_pt4:

acw4_loop:
lda $2ea606     // the value at this address will be 0x39 if tsushiy's patch has been applied
cmp #$39
bne +

lda $288040,x   // if we're here, we're using tsushiy's patch, so use alternate action strings
jmp acw4_store

+               
lda $2ba070,x   // if we're here, we're using the original patch, so use original strings

acw4_store:
sta $74fd,y
cmp #$ff
beq acw4_leave
iny
inx
cpy #$0013
bne acw4_loop

acw4_leave:
jsl strlen
dec
sta $0754
rtl


//-----------------------------------------------------------------------------
// this function copies expanded item names for use in the casting window
autoformat_casting_window_pt5:
ldx $2a
lda $2b8030,x
cmp #$B2
bne acw5_loop
inx

acw5_loop:
lda $2b8030,x
sta $74fd,y
cmp #$ff
beq acw5_leaveloop
iny
inx
cpy #$0013
bne acw5_loop

acw5_leaveloop:
jsl strlen
sta $0754
rtl


//-----------------------------------------------------------------------------
// this function prepares the proper size casting window for action text
// like Steal, Cover, etc.
autoformat_casting_window_pt6:
lda $2E8160,x
sta $74fd,y
cmp #$ff
beq +
iny
inx
cpy #$0007
bne autoformat_casting_window_pt6

+
jsl strlen
sta $0754
rtl


//-----------------------------------------------------------------------------
// allows expanded item names in "___ stolen!" text, also displays icons as needed
// limited window space means some long names will get cut, though
expand_stolen_item_names:
ldy #$0000
ldx $2a
lda $2b8030,x
cmp #$B2  // we'll only skip the first character (icon) if it's blank
bne +
inx

+
rtl


//-----------------------------------------------------------------------------
// Display static handedness strings in battle item menu
// We need to bake these strings in rather than use the original game's
// dynamic programming which isn't too helpful, is limited by VRAM, etc.
write_battle_handedness_strings:
pha
phy

cmp #$06       // If the sixth battle window has been created, let's fill it with our text
bne +          // otherwise proceed as normal

ldy #$0000     // initialize our counter

-
tyx
lda $2BF2E0,x  // load the next byte in our custom string
cmp #$00       // if it's 0 then let's leave this loop
beq +

pha
tya
asl
tax
pla
sta $7ed22c,x  // store our new letter in the proper spot in RAM before it's sent to VRAM

iny
jmp -

+
ply
pla
rtl


//-----------------------------------------------------------------------------
strlen:
phx         // save value of X, Y, and the word at $00
phy
ldx $00
phx

ldx $ef50   // grab pointer to start of string
stx $00     // plop the pointer in memory for easy use
ldy #$0000  // initialize our length count

lda ($00)   // sometimes a string might start with a control code and an argument, so let's compensate for that
cmp #$10
bcs strlen_loop
dey

strlen_loop:
lda ($00)
cmp #$ff
beq strlen_end
cmp #$00
beq strlen_end
ldx $00
inx
stx $00
iny
cpy #18     // I've made it so the expanded strings can only be 18 letters max, could be moved to 32 if needed
beq strlen_end
jmp strlen_loop

strlen_end:
plx         // restore the word at $00 to what it was beforehand
stx $00

lda #$00
xba
tya         // A now holds the length of the expanded string!
ply         // get our initial Y back
plx         // get our initial X back too
rtl       