//====================================================================
// WAR ROOM TUTORIAL SIGN ADDITION
//====================================================================
// Everything in this file adds a sign to the War Room that explains
// stuff about the game. When I first played the Japanese hack, I was
// unfamiliar with the original Final Fantasy II for Famicom and was
// constantly unsure of what to do/where to go next. So to prevent
// other players from getting stuck in the same way, I decided to add
// a sign in the first room of the game explaining that you should
// regularly visit Princess Hilda after finishing a mission or when
// unsure of what to do next in general.
//
// These modifications are kinda messy because I'm doing stuff
// manually and force-inserting stuff. Adding this sign in a standad
// waywould be very difficult, impossible, or require changes/cuts to
// other things in the game. Doing this brute force method means that
// some of these changes won't show up in any FF4 editing tools.
// If you were to load this translation hack in a map editor, for
// example, there'd be no evidence that this added sign even exists.
//
// At first, I wasn't really sure where to stick this new sign, but
// then I realized that the prayer room is idela because there are two
// NPC soldiers in the room that say the same thing everytime, meaning
// one is redundant. This file replaces the redundant NPC with a
// custom NPC for our needs, and moves a separate soldier into the
// original soldier's place. It all works out really well, especally
// given the fact that this is the first and maybe most important room
// in the game.
//
// After doing all the nonsense technical stuff, the sign will display
// whatever is in Main Text Bank 1, Line 243.


//-----------------------------------------------------------------------------
// Modify the prayer room map layout to put a "wall switch" tile where we want our poster to be
org $179f9c; db $81,$01,$40,$28

// Modify the map graphics loader to overwrite the "wall switch" tile with our custom sign tile graphics
// I chose the wall switch tile because it shares a palette with the surrounding tiles
// Our custom routine will only do this when the prayer room is being loaded
org $15b0b2; fill $0B,$EA
org $15b0b2; jsl display_new_poster_tile
org $2BFFA0; db "****************NEW GRAPHICS: WAR ROOM POSTER"
org $2BFFD0; incbin bin/graphics-castle-poster-tile.bin    // store our custom "tutorial sign" tile in ROM

// Move existing NPC data around and overwrite a redudant NPC with a standard "wall poster NPC"
org $1384ba; db $5D,$04,$0A,$76   // replace redundant soldier NPC with NPC 0x5D (wall poster)
org $1384c2; db $fd,$04,$0c,$76   // move third soldier down to where the first soldier used to be

// Make generic "wall poster NPC" load our custom script instead of original "display line 4" script
org $e1f6; fill $0e,$EA
org $e1f6; jsl add_poster_to_prayer_room

// This is our custom script code for our custom poster, displays Line #243 of Bank 1
org $12e690; db $f0,$f3,$ff   


//====================================================================
// CUSTOM CODE THAT NEEDS TO BE STORED IN EXPANDED ROM
//====================================================================

org $2BFD00;
db "****************NEW CODE: WAR ROOM POSTER"

//-----------------------------------------------------------------------------
add_poster_to_prayer_room:
phx
lda $0fde             // load current map ID
cmp #$17              // if not currently on map 0x17 (the prayer room)
bne orig_poster_code  // then do original code

                      // if we're here, we're in the prayer room
ldx $06bc             // now we do a check to make sure we're really
cpx #$0203            // checking the poster and not the problematic NPC
bne orig_poster_code  // now check to see if we're reading the poster

plx
lda #$90              // tell the game to load our custom script code
sta $09d3             // at offset 0x6490, which is address 12e690
lda #$64
sta $09d4
rtl

orig_poster_code:
plx
lda $128000,x         // original code
sta $09d3
lda $128001,x
sta $09d4
rtl

//-----------------------------------------------------------------------------
display_new_poster_tile:
lda $15b104,x    // orig code
sta $4304
jsl $15b143

lda $0fde        // get current Map ID
cmp #$17         // if Map ID is 0x17, the prayer room, do custom stuff
bne +            // else skip our custom stuff

// If we're here, we're in the prayer room
// HDMA our new "tutorial sign" tile over the "wall switch" tile in VRAM
stz $420b    // initialize stuff
stz $2115
stz $4300
lda #$2b     // set the source bank we'll be using
sta $4304
ldx #$ffd0   // set the source offset we'll be using
stx $4302
lda #$18     // set the offset thingy we'll be using?
sta $4301
ldx #$0860   // set the VRAM target
stx $2116
ldy #$0000   // initialize our counter, we only want to do 4 tiles

-
lda #$80
sta $2115
lda #$01
sta $4300
ldx #$0010   // number of bytes to transfer
stx $4305
lda #$01
sta $420b    // activate transfer

stz $420b
stz $2115
stz $4300

ldx #$0008   // number of bytes to transfer
stx $4305
lda #$01
sta $420b    // activate transfer

iny
cpy #$0002   // loop again if less than 4 tiles have been transferred
bne -

stz $420b
stz $2115
stz $4300


+
rtl