//====================================================================
// TREASURE CHEST COUNTER
//====================================================================
// This code makes it so that when you open the main menu, you will
// see a count of how many treasure chests and hidden items you have
// uncovered. It does this by checking the treasure chest trigger
// flags found in $12a0 through $12df and then totaling up the number
// of set bits. These flags are inherent to the original game, so no
// extra stuff needs to be saved in SRAM.
//
// Other parts of this code display the treasure counter during the
// end credits sequence, along with the clear time.


//----------------------------------------------------------------------------------
// MAIN MENU MODIFICATIONS
org $1db69; db $ea,$04,$08,$07  // move and resize time/gil window

org $187cf; dw $80AC            // use the "Gil" icon instead of the full string
org $187d2; dw $05ac            // move Gil icon up and left a bit
org $187db; dw $05ec            // move gil digit display

org $187c6; dw $80AE            // use the clock icon instead of the "Time" string
org $187c9; dw $056c            // move clock icon on main menu
org $18bc4; dw $0578            // move numerical time on main menu
org $18bc8; dw $057a
org $18bd6; dw $0570
org $18bdb; dw $0572
org $18be0; dw $0574
org $18be5; dw $0576

org $187da; fill $09,$EA
org $187da; jsr add_treasures_info



//----------------------------------------------------------------------------------
// END CREDITS MODIFICATIONS

org $13d680
jsl display_credits_final_stats
nop




//==================================================================================
// New code added to original program banks
// This code is used to do things that would be hard/impossible by expanding into
// new banks, but there's very little room to maneuver in here
// The roughly free area (assumedly?) begins at $1ff48 and ends at $1ffff
// We've already used a bit of this space in menus.asm so we'll try to avoid it
//==================================================================================
org $1ff90

//-------------------------------------------------------------------------------
add_treasures_info:
ldy #$80AA
ldx #$062c
jsr $82cd
ldy #$066c
jsl display_treasures_found
rts





//====================================================================
// EXPANDED AREA CODE GOES HERE
//====================================================================

org $2BFB00
db "****************NEW CODE: TREASURE COUNTER (MENU & CREDITS)"

//-------------------------------------------------------------------------------
display_numerals:
sep #$20
ldx $41
-
lda $0163,x
sta ($29),y
dey

pha
lda $0760   // check if we're using 1-byte or 2-byte text format
bne +
dey

+
pla
inx
cpx $0750
bne -
rtl


//-------------------------------------------------------------------------------
display_treasures_found:
ldx #$0003    // tell the display routine to only display 3 digits
stx $0750
ldx #$0000    // tell the display routine to use 2-byte character format
stx $0760

jsl count_treasures_found

phy
phy
jsl convert_treasures_found_to_text

rep #$20
pla
clc
adc #$000e
tay
jsl display_numerals

ply
sep #$20
ldy #$05ec    // original overwritten code
lda $16a2
ldx $16a0
rtl


//-------------------------------------------------------------------------------
convert_treasures_found_to_text:
sep #$20
stx $73
sta $75
lda #$ff
stz $41
stz $42
ldx $41

-
sta $0163,x
inx
cpx $0750
bne -

ldy $41

-
jsr do_number_to_text_conversion
lda $1d
ora #$80
sta $0163,y
iny
lda $73
ora $74
ora $75
beq +
cpy $0750
bne -

+
rtl

//-------------------------------------------------------------------------------

do_number_to_text_conversion:
stz $1d
ldx #$0018
rol $73
rol $74
rol $75

-
rol $1d
lda $1d
cmp #$0a
bcc +
sbc #$0a
sta $1d
+
rol $73
rol $74
rol $75
dex
bne -

rts


//-------------------------------------------------------------------------------
count_treasures_found:
phy

sep #$20          // store the flag byte we'll be looking at in RAM
ldx #$0000        // initialize our big loop counter
ldy #$0000        // we'll be reusing this area that usually gets used by the main text
sty $0752         // 0x752 will be our chest counter, so let's initialize it to 0
sty $0754         // 0x754 will be where we do some bit mathy stuff

chest_flag_main_loop_start:
lda $12a0,x
sta $0754
lda #$01          // initialize our bit
sta $0755         // 0x755 is where we'll temporarily store our bit
ldy #$0008        // initialize our bit loop counter

chest_flag_bit_loop_start:
and $0754         // is the current flag in question is set?
beq +

phy               // ...if so, increment treasure counter
ldy $0752
iny
sty $0752
ply

+
dey
beq +

lda $0755         // move our bit to the left one for the next bit check
asl               
sta $0755
jmp chest_flag_bit_loop_start

+
inx
cpx #$0040
bne chest_flag_main_loop_start

rep #$20          // now we get ready to leave the counting function
lda #$0000        // A acts as the highest byte in the three-byte result count, so let's just set it to 0
ldx $0752         // X now has the total number of chests opened
ply
rtl



//-------------------------------------------------------------------------------
display_credits_final_stats:

pha
phy

cpy #$0010     // a crude and unreliable way of telling if we're loading text for the outer space legend or the end credits
beq +          // if we're doing the space legend, let's just skip all this treasure nonsense

ldx #$0001     // tell the display routine to use 1-byte character format
stx $0760
ldx #$0003     // tell the display routine to only display 3 digits
stx $0750
jsl count_treasures_found
jsl convert_treasures_found_to_text

sep #$20
ldx #$8000     // designate where we want to write the chest total in RAM
stx $29
lda #$7e
sta $2b
ldy #$13ae     // this is the offset from $7e8000, change if credits change
jsl display_numerals
jsl display_clear_time_in_credits

+
ply
pla
stz $95
ldx #$8000
rtl
