//====================================================================
// MAIN DIALOGUE AND MAIN DIALOGUE WINDOW MODIFICATIONS
//====================================================================

org $298400; db "****************MAIN TEXT: BANK 1, continues into next ROM bank too"
org $2C8000; db "****************MAIN TEXT: BANK 2a"
org $2D8000; db "****************MAIN TEXT: BANK 2b"
org $2EAD00; db "****************MAIN TEXT: BANK 2c"
org $2FA000; db "****************MAIN TEXT: BANK 3"
org $2B8000; db "****************NEW TEXT: LONG ITEM NAMES"


//-----------------------------------------------------------------------------
// allow for expanded main text banks, three banks in all
org $b4f5; jsl banksplitter1
org $b3e4; jsl banksplitter2
org $b4ea; jsl banksplitter2
org $b3ea; jsl banksplitter2b
org $b3f2; jsl banksplitter2b
org $b4fc; lda $2FA030,x       // Bank 3


//-----------------------------------------------------------------------------
// remove auto-double ellipses thing found in original ff4j script routine
org $b4a2; nop; nop; nop; nop; nop; nop; nop;


//-----------------------------------------------------------------------------
// allow item icons to be displayed in the main script via the [07] control code
org $b586; jsl displayitemicon; nop


//-----------------------------------------------------------------------------
// allow for expanded item names in main script display
org $b58d; lda $2B8030,x
org $b56d; jsl expanditems; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;


//-----------------------------------------------------------------------------
// MAKE FULL USE OF SCRIPT TEXT BOX, TAKES OUT ORIGINAL LEFT & RIGHT PADDING
// MIGHT STILL BE KINDA BUGGY, CAN'T SEEM TO USE ALL 104 CHARS PER
// WINDOW AT ONCE, SOMETIMES THE LAST 6-10 WILL GO UNDISPLAYED

// we need to expand the screen text by 0x20 bytes, so we pull everything up by 0x20
// move start of block from 0x772 to 0x752
org $b3ff; db $52
org $b42f; db $52
org $b4bc; db $52
org $b4c0; db $52
org $b4dd; db $52
org $b4e1; db $52
org $b4e4; db $52
org $b416; db $52
org $b448; db $52

// move all 0x774 references back by 0x20 to 0x754
org $b4ad; db $54
org $b54f; db $54
org $b599; db $54
org $b5d5; db $54
org $b610; db $54
org $b627; db $54
org $b649; db $54
org $b65e; db $54
org $b506; db $54
org $b509; db $55

// move all 0x834 references back by 0x10
org $b4b1; db $24
org $b50d; db $24
org $b510; db $25
org $b553; db $24
org $b59d; db $24
org $b5da; db $24
org $b613; db $24
org $b62a; db $24
org $b64c; db $24
org $b661; db $24

// modify dma routines to find new window text locations
org $b828; db $54
org $b82d; db $24

// move text left one column
org $b81d; db $03	

// increase line lengths by 2, so 0x1A letters per line
org $b86c; db $1A
org $b88b; db $1A
org $b62e; db $1A
org $b633; db $34
org $b638; db $4E
org $b63d; db $68
org $b4c5; db $68
org $b617; db $68
org $b650; db $D0
org $b665; db $D0

// we need to modify how the original game handles newlines when at the maximum line length
// redirect original newline handler to our new custom stuff
org $b626
jsl newlinefix
nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;




//====================================================================
// NEW CUSTOM CODE GOES HERE
//====================================================================

//-----------------------------------------------------------------------------
// here's our custom newline handler, which is mostly the same as the original
org $2BF300
db "****************NEW CODE: MAIN TEXT IMPROVEMENTS"

newlinefix:
// if we're already at the edge of a line, let's just skip all the original code
cpy #$001A
beq newlineleave
cpy #$0034
beq newlineleave
cpy #$004E
beq newlineleave
cpy #$0068
beq newlineleave

// here's the old, original code
lda #$ff
newlineloop:
sta $0754,y
sta $0824,y
iny
cpy #$001A
beq newlineleave
cpy #$0034
beq newlineleave
cpy #$004e
beq newlineleave
cpy #$0068
bne newlineloop

newlineleave:
rtl


//-----------------------------------------------------------------------------
// custom code for adding/skipping item icons in the main script
displayitemicon:
lda $2b8030,x   // load first character of long item name
cmp #$B2        // if first character is 0xB2 (a custom blank), don't display it
bne +
inx

+
ldy $3d         // original routine's code
lda #$1F        // allow for longer names
rtl


//-----------------------------------------------------------------------------
// we're allowing for extra long item names in the main script text, so we need to modify how these strings are located
// each item name entry is 0x20 bytes instead of 9 bytes
expanditems:
stz $19
asl; rol $19 // id * 0x02
asl; rol $19 // id * 0x04
asl; rol $19 // id * 0x08
asl; rol $19 // id * 0x10
asl; rol $19 // id * 0x20
sta $18
rtl


//-----------------------------------------------------------------------------
// Move and expand main text banks, there are 3 banks total that need to be
// dealt with

banksplitter1:    // if x < 0x8000 then do normal load, if >= 0x8000, then load from diff source spot
cpx #$8000
bcs +
LDA $298000,x    // if < 0x8000 load from the first area
jmp byebye1

+                // we're gonna go into the new second area, so we need to adjust the offset
rep #$20         // set accumulator to 16-bit

phx
txa
sbc #$8000
tax
lda #$0000
sep #$20
LDA $2A8000,x
plx

byebye1:
pha
pla
rtl


//-----------------------------------------------------------------------------
banksplitter2:

sep #$20         // we want to preserve what's in B, map problems happen otherwise
xba
pha
xba

lda $1701
xba
lda $1702        
rep #$20         // now we have the full line number

cmp #$0040       // if line number <= 0x3F then load from Sub Bank 0
bcc subbank0
cmp #$0120       // if line number >= 0x40 and < 0x11F then load from Sub Bank 1
bcc subbank1
jmp subbank2     // else load from Sub Bank 2

subbank0:
sep #$20
lda $2C8031,x
xba              // restore B
pla
xba
rtl

subbank1:
sep #$20
lda $2D8031,x
xba              // restore B
pla
xba
rtl

subbank2:
sep #$20
lda $2EAD31,x
xba              // restore B
pla
xba
rtl
				 

//-----------------------------------------------------------------------------
banksplitter2b:
sep #$20         // we want to preserve what's in B, map problems happen otherwise
xba
pha
xba

lda $1701
xba
lda $1702        
rep #$20         // now we have the full line number

cmp #$0040       // if line number <= 0x3F then load from Sub Bank 0
bcc subbank0b
cmp #$0120       // if line number >= 0x40 and < 0x11F then load from Sub Bank 1
bcc subbank1b
jmp subbank2b     // else load from Sub Bank 2

subbank0b:
sep #$20
lda $2C8030,x
xba              // restore B
pla
xba
rtl

subbank1b:
sep #$20
lda $2D8030,x
xba              // restore B
pla
xba
rtl

subbank2b:
sep #$20
lda $2EAD30,x
xba              // restore B
pla
xba
rtl