﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Text.RegularExpressions;

namespace FF2Remake
{
    class Program
    {
        static List<string> ff2eBank1;
        static List<string> ff2eBank2;
        static List<string> ff2eBank3;
        static TableReader eTable;

        static void Main(string[] args)
        {
            Console.OutputEncoding = System.Text.Encoding.Unicode;

            ff2eBank1 = new List<string>();
            ff2eBank2 = new List<string>();
            ff2eBank3 = new List<string>();

            string[] romNames = { "ff2remake-j.smc", "ff2remake-j-tsushiy.smc" };
            string[] newRomNames = { "ff2remake-e.smc", "ff2remake-e-alt-enemies.smc" };
            string[] introTextFiles = { "intro-screen.txt", "intro-screen-alt.txt" };
            eTable = new TableReader("table-ff2re-e.txt");
            int i = 0;

            // This cycles through the two different versions to build both remake versions
            //
            // If you want to modify these tools to do FF4 translations or other stuff, remove the surrounding for loop
            // or edit the romNames list and newRomNames list above as needed for your own project
            for (i = 0; i < romNames.Length; i++)
            {
                string romName = romNames[i];
                string newRomName = newRomNames[i];

                // we don't want to edit the original files directly, so let's make a copy first
                File.Delete(newRomName);
                File.Copy(romName, newRomName);
                FileStream romFile = new FileStream(newRomName, FileMode.Open);

                // we only need to expand the first ROM, the alternate enemies ROM is already expanded
                if (i == 0)
                {
                    romFile.Seek(0x1FFFFF, SeekOrigin.Begin);
                    romFile.WriteByte(0x00);
                }
                romFile.Close();

                // now let's do all the text writing
                WriteEnemiesLong(newRomName, "text/battle_enemy_names.txt", 0x15B7E0, eTable);   // standard enemy names
                WriteEnemiesLong(newRomName, "text/battle_enemy_names-alt.txt", 0x15D400, eTable);   // enemy names when tsushiy's enemy patch is applied
                WriteItems(newRomName, eTable);
                WriteItemsLong(newRomName, eTable);
                WriteSpells(newRomName, "text/spell_names.txt", 0x1420f0, eTable);
                WriteSpells(newRomName, "text/spell_names-alt.txt", 0x1422d0, eTable);
                WriteSpellsLong(newRomName, "text/spell_names_long.txt", 0x170930, eTable);
                WriteSpellsLong(newRomName, "text/spell_names_long-alt.txt", 0x1417c0, eTable);

                WriteNames(newRomName, eTable);
                WriteClasses(newRomName, eTable);
                WriteMenus(newRomName, eTable);
                WriteBattleHandednessStrings(newRomName, eTable);
                WriteBattleCommandsLong(newRomName, eTable);
                WriteBattleActionsLong(newRomName, "text/battle_actions.txt", 0x15A070, eTable);  // standard action names
                WriteBattleActionsLong(newRomName, "text/battle_actions-alt.txt", 0x140040, eTable);  // modified action names to match tsushiy's patch
                WriteLocations(newRomName, eTable);
                WriteBattleDialogue(newRomName, eTable);
                WriteStatuses(newRomName, eTable);
                WriteItemDescriptions(newRomName, eTable);
                WriteBattleNotices(newRomName, eTable);
                WriteBank1Text(newRomName, eTable);
                WriteBank2Text(newRomName, eTable);
                WriteBank3Text(newRomName, eTable);
                WriteCredits(newRomName, eTable);
                WriteOuterSpaceLegend(newRomName, eTable);

                WriteIntroScreenText(newRomName, "text/" + introTextFiles[i], 0x144400);
            }
        }

        //=================================================================================================

        static void WriteEnemiesLong(string newRomName, string filename, int loc, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] enemies = System.IO.File.ReadAllLines(filename);
            for (int i = 0; i < 224; i++)
            {
                string newName = enemies[i].Trim();
                newName = newName.Replace(" ", "[B2]");
                eTable.WriteLine(newName, output, loc + i * 0x20, 0x20, 0xFF);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteItems(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] items = System.IO.File.ReadAllLines("text/item_names.txt");
            for (int i = 0; i < 256; i++)
            {
                string newName = items[i].TrimEnd();
                eTable.WriteLine(newName, output, 0x78000 + i * 9, 9, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteItemsLong(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] items = System.IO.File.ReadAllLines("text/item_names_long.txt");
            for (int i = 0; i < 256; i++)
            {
                string newName = items[i].TrimEnd();
                newName = newName.Replace(" ", "[B2]");
                eTable.WriteLine(newName, output, 0x158030 + i * 32, 32, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteSpells(string newRomName, string filename, int loc, TableReader table)
        {
            //0x78900 original loc
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] items = System.IO.File.ReadAllLines(filename);
            for (int i = 0; i < 72; i++)
            {
                string newName = items[i].TrimEnd();
                eTable.WriteLine(newName, output, loc + i * 6, 6, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteSpellsLong(string newRomName, string filename, int loc, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] items = System.IO.File.ReadAllLines(filename);
            for (int i = 0; i < 72; i++)
            {
                string newName = items[i].TrimEnd();
                newName = newName.Replace(" ", "[B2]");
                eTable.WriteLine(newName, output, loc + i * 32, 32, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteNames(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] names = System.IO.File.ReadAllLines("text/character_names.txt");
            int maxLength = 6;

            for (int i = 0; i < 28; i++)
            {
                string newName = names[i].Trim();
                if (newName.Length > maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }
                eTable.WriteLine(newName, output, 0x7A710 + i * maxLength, maxLength, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================
        
        static void WriteClasses(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] names = System.IO.File.ReadAllLines("text/character_classes.txt");
            int maxLength = 16;

            for (int i = 0; i < names.Length; i++)
            {
                string newName = names[i].Trim();
                if (newName.Length > maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }
                eTable.WriteLine(newName, output, 0x170030 + i * maxLength, maxLength, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteMenus(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] menuDataStrings = System.IO.File.ReadAllLines("text/menus.txt");

            int pointerRegion = 0x178030;
            int textRegion = 0x178230;
            int offset = 0;

            output.Seek(textRegion, SeekOrigin.Begin);
            for (int i = 0; i < menuDataStrings.Length; i++)
            {
                string tempStr = menuDataStrings[i].Trim();

                int temp = (int)(output.Position - pointerRegion + 0x8030);
                output.Seek(pointerRegion + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                int length = eTable.WriteLine(tempStr, output, textRegion + offset);

                offset += (length);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBattleHandednessStrings(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] strings = System.IO.File.ReadAllLines("text/battle_handedness.txt");
            string[] splitter = { ":" };

            for (int i = 0; i < strings.Length; i++)
            {
                string[] substrings = strings[i].Split(splitter, 3, StringSplitOptions.None);
                if (substrings.Length < 3)
                {
                    Console.WriteLine("One or more missing ':' in battle_handedness.txt, skipping all further entries");
                    break;
                }

                int loc = SNESLoROMtoHex(Convert.ToInt32(substrings[0], 16));
                int maxLength = Convert.ToInt32(substrings[1]);
                if (maxLength <= 0)
                {
                    maxLength = -1;
                }

                if (loc != -1)
                {
                    eTable.WriteLine(substrings[2].Trim(), output, loc, maxLength);
                }
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBattleCommandsLong(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] menus = System.IO.File.ReadAllLines("text/battle_commands.txt");
            for (int i = 0; i < menus.Length; i++)
            {
                string newName = menus[i].Trim();
                eTable.WriteLine(newName, output, 0x170160 + i * 7, 7, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteBattleActionsLong(string newRomName, string filename, int loc, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] actions = System.IO.File.ReadAllLines(filename);
            for (int i = 0; i < actions.Length; i++)
            {
                string newName = actions[i].Trim();
                if (newName.Length > 32)
                {
                    newName = newName.Substring(0, 32);
                }
                newName = newName.Replace(" ", "[B2]");
                eTable.WriteLine(newName, output, loc + i * 32, 32, 0xFF);
            }
            output.Close();
        }

        //=================================================================================================

        static void WriteLocations(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] locations = System.IO.File.ReadAllLines("text/location_names.txt");

            int startingLoc = 0x170280;
            int offset = 0;
            int maxLength = 0x1A;

            for (int i = 0; i < 121; i++)
            {
                string newName = locations[i].Trim();
                if (newName.Length >= maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }
                
                eTable.WriteLine(newName, output, startingLoc + offset);
                offset += (newName.Length + 1);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBattleDialogue(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] messages = System.IO.File.ReadAllLines("text/battle_dialogue.txt");

            int pointerArea = 0x171a40;
            int textArea = 0x171c00;
            int offset = 0;
            int maxLength = 28;

            int[] pointers = new int[186];
            for (int i = 0; i < 186; i++)
            {
                string newName = messages[i].Trim();
                if (newName.Length > maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }

                pointers[i] = textArea + offset;
                eTable.WriteLine(newName, output, pointers[i]);

                int temp = (textArea & 0xFFFF) + 0x8000 + offset;
                output.Seek(pointerArea + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (newName.Length + 1);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteStatuses(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] statuses = System.IO.File.ReadAllLines("text/statuses.txt");

            int pointerArea = 0x171340;
            int textArea = 0x1713C0;
            int offset = 0x00;
            int maxLength = 10;
            int[] pointers = new int[32];

            for (int i = 0; i < 32; i++)
            {
                string newName = statuses[i].Trim();
                if (newName.Length > maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }

                pointers[i] = textArea + offset;
                eTable.WriteLine(newName, output, pointers[i]);

                int temp = (pointers[i] & 0xFFFF) + 0x8000;
                output.Seek(pointerArea + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (newName.Length + 1);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteItemDescriptions(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string textBlob = System.IO.File.ReadAllText("text/item_descriptions.txt");
            string[] splitter = { "\n----------\n" };
            string[] descriptions = textBlob.Split(splitter, StringSplitOptions.None);

            int pointerArea = 0x154040;
            int textArea = 0x154280;
            int offset = 0;
            int newlineAddress = 0x14;

            for (int i = 0; i < 256; i++)
            {
                string newName = descriptions[i].Trim();

                // handle the newlines, which require some weird stuff
                newName = newName.Replace("\r", "");
                newlineAddress = 0x12;
                for (int j = 0; j < 2; j++) // only allow 3 lines
                {
                    int newlinePos = newName.IndexOf("\n");
                    if (newlinePos >= 0)
                    {
                        newlineAddress += 0x40;
                        newName = newName.Substring(0, newlinePos) + "[01][" + newlineAddress.ToString("X2") + "][00]" + newName.Substring(newlinePos + 1);
                    }
                }   

                // store pointer offset for this item description
                output.Seek(pointerArea + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(offset & 0xFF));
                output.WriteByte((byte)((offset >> 8) & 0xFF));

                // store the item description
                eTable.WriteLine(newName, output, textArea + offset);

                offset += (newName.Length + 1);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBattleNotices(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string[] battleTexts = System.IO.File.ReadAllLines("text/battle_notices.txt");

            int pointerArea = 0x1714e0;
            int textArea = 0x1715A0;
            int offset = 0;
            int maxLength = 28;
            int[] pointers = new int[59];

            for (int i = 0; i < 59; i++)
            {
                string newName = battleTexts[i].Trim();
                if (newName.Length > maxLength)
                {
                    newName = newName.Substring(0, maxLength);
                }

                pointers[i] = textArea + offset;
                int convertedLength = eTable.WriteLine(newName, output, pointers[i]);

                int temp = (textArea & 0xFFFF) + 0x8000 + offset;
                output.Seek(pointerArea + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (convertedLength);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBank1Text(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string textBlob = System.IO.File.ReadAllText("text/maintext1.txt");
            string[] splitter = { "\n----------\n" };
            string[] bank1Lines = textBlob.Split(splitter, StringSplitOptions.None);

            int[] pointers = new int[5000];
            int pointerRegion     = 0x80000;
            int startingLoc       = 0x148000;
            int offset = 0x450;

            for (int i = 0; i < bank1Lines.Length; i++)
            {
                string newText = bank1Lines[i];

                for (int p = 0; p < 10; p++)
                {
                    newText = newText.Replace("\r", "");
                    newText = newText.Replace("\n\n\n\n", "\n \n \n \n");
                    newText = newText.Replace("\n\n\n", "\n \n \n");
                    newText = newText.Replace("\n\n", "\n \n");
                    newText = newText.Replace("\n", "[01]");
                    newText = newText.Replace("_", "]["); // gotta undo this to make control codes work right
                    newText = newText.Replace("[FB]", "[04][01]");
                    newText = newText.Trim();
                }

                newText = FormatMainTextLine(newText, eTable, true, false);

                pointers[i] = startingLoc + offset;
                int convertedLength = eTable.WriteLine(newText, output, pointers[i]);

                int temp = offset;
                output.Seek(pointerRegion + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.Seek(pointerRegion + i * 2 + 1, SeekOrigin.Begin);
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (convertedLength);
            }

            // we've now finished dealing with Sub Bank 2, and if we've written beyond the boundary, let's make a note of it
            if (offset >= 0x10000)
            {
                int overflowAmount = offset - 0x10000;
                Console.WriteLine("NOTE!!! Bank 1 text has exceeded its limit.\nReduce Bank 1 text by " + overflowAmount.ToString() + " bytes to ensure all text works properly!\n\n");
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBank2Text(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string textBlob = System.IO.File.ReadAllText("text/maintext2.txt");
            string[] splitter = { "\r\n----------\r\n" };
            string[] bank2Lines = textBlob.Split(splitter, StringSplitOptions.None);

            int[] pointers = new int[5000];
            int pointerRegion = 0x88000;
            int[] startingLoc = { 0x160031, 0x168031, 0x172D31 };
            int offset = 0;

            int currSubBank = 0;   // we're gonna divide Bank 2 text into three sub banks to expand the amount of available text without worry
            for (int i = 0; i < bank2Lines.Length; i++)
            {
                string newText = bank2Lines[i];

                for (int k = 0; k < 10; k++)
                {
                    newText = newText.Replace(" [FC]", "[FC]");
                    newText = newText.Replace("[FC] ", "[FC]");
                    newText = newText.Replace("\r\n[FC]", "[FC]");
                    newText = newText.Replace("[FC]\r\n", "[FC]");
                }

                for (int p = 0; p < 10; p++)
                {
                    newText = newText.Replace("\r", "");
                    newText = newText.Replace("\n\n\n\n", "\n \n \n \n");
                    newText = newText.Replace("\n\n\n", "\n \n \n");
                    newText = newText.Replace("\n\n", "\n \n");
                    newText = newText.Replace("\n", "[01]");
                    newText = newText.Replace("_", "]["); // gotta undo this to make control codes work right
                    newText = newText.Replace("[FB]", "[04][01]");
                }

                newText = newText.Trim();
                newText = FormatMainTextLine(newText, eTable, false, false);

                if (i == 0x40)            // switch from Sub Bank 0 to Sub Bank 1 after dealing with the first 0x40 lines
                {
                    currSubBank = 1;
                    offset = 0;
                }
                else if (i == 0x120)   // switch from Sub Bank 1 to Sub Bank 2 after dealing with lines up until Line 0x120
                {
                    currSubBank = 2;
                    offset = 0;
                }

                pointers[i] = startingLoc[currSubBank] + offset;

                int convertedLength = eTable.WriteLine(newText, output, pointers[i]);

                int temp = offset;
                output.Seek(pointerRegion + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.Seek(pointerRegion + i * 2 + 1, SeekOrigin.Begin);
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (convertedLength);
            }

            // we've just finish handling Sub Bank 3, so if we've written beyond the sub bank boundaries, make a note of it to the user
            // for FF2c, we're putting Sub Bank 3 in a smaller area then usual so let's compensate
            if (offset >= 0x5300)
            {
                int overflowAmount = offset - 0x5300;
                Console.WriteLine("NOTE!!! Bank 2c text has exceeded its limit.\nReduce Bank 2 text by " + overflowAmount.ToString() + " bytes to ensure all text works properly!\n\n");
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteBank3Text(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
            string textBlob = System.IO.File.ReadAllText("text/maintext3.txt");
            string[] splitter = { "\n----------\n" };
            string[] bank3Lines = textBlob.Split(splitter, StringSplitOptions.RemoveEmptyEntries);

            int[] pointers = new int[5000];
            int pointerRegion = 0x9A500;
            int startingLoc = 0x17A030;
            int offset = 0;

            for (int i = 0; i < 256; i++)
            {
                string newText = bank3Lines[i];

                for (int p = 0; p < 10; p++)
                {
                    newText = newText.Replace("\r", "");
                    newText = newText.Replace("\n\n\n\n", "\n \n \n \n");
                    newText = newText.Replace("\n\n\n", "\n \n \n");
                    newText = newText.Replace("\n\n", "\n \n");
                    newText = newText.Replace("\n", "[01]");
                    newText = newText.Replace("_", "]["); // gotta undo this to make control codes work right
                }
                
                newText = newText.Trim();
                newText = FormatMainTextLine(newText, eTable, false);

                pointers[i] = startingLoc + offset;
                int convertedLength = eTable.WriteLine(newText, output, pointers[i]);

                int temp = offset;
                output.Seek(pointerRegion + i * 2, SeekOrigin.Begin);
                output.WriteByte((byte)(temp & 0xFF));
                output.Seek(pointerRegion + i * 2 + 1, SeekOrigin.Begin);
                output.WriteByte((byte)((temp >> 8) & 0xFF));

                offset += (convertedLength);
            }

            output.Close();
        }

        //=================================================================================================

        static void WriteCredits(string newRomName, TableReader table)
        {
            FileStream output = new FileStream(newRomName, FileMode.Open);
 
            int startingLoc = 0x17F630;

            string credits = System.IO.File.ReadAllText("text/credits.txt");
            credits = credits.Replace("\r", "");
            credits = credits.Replace("\n", "[01]");

            table.WriteLine(credits, output, startingLoc);

            output.Close();   
        }


        static void WriteOuterSpaceLegend(string newRomName, TableReader table)
        {
            string legend = System.IO.File.ReadAllText("text/outer-space-legend.txt");
            int startingLoc = 0x1424c0;

            FileStream output = new FileStream(newRomName, FileMode.Open);
            legend = legend.Replace("\r", "");
            legend = legend.Replace("\n", "[01]");
            table.WriteLine(legend, output, startingLoc);
            output.Close();
        }


        static void WriteIntroScreenText(string newRomName, string filename, int loc)
        {
            TableReader introTable = new TableReader("table-ff2re-e-intro.txt");
            FileStream output = new FileStream(newRomName, FileMode.Open);
            byte currPal = 0;

            string credits = System.IO.File.ReadAllText(filename);
            credits = credits.Replace("\r", "");

            // initialize the intro screen text area with 00s
            output.Seek(loc, SeekOrigin.Begin);
            for (int i = 0; i < 0x400; i++ )
            {
                output.WriteByte(0x01);
                output.WriteByte(0x00);
            }

            int xIndent = 2;
            int yIndent = 2;

            int x = xIndent;
            int y = yIndent;

            for (int i = 0; i < credits.Length; i++)
            {
                int outputChar = 0x00;
                if (credits[i] == '*')
                {
                    currPal += 16;
                    if (currPal > 16)
                    {
                        currPal = 0;
                    }
                }
                else if (credits[i] == '\n')
                {
                    x = xIndent;
                    y += 2;
                }
                else
                {
                    outputChar = introTable.EntryToHex(credits[i].ToString());
                    int writeLoc = loc + (x * 2) + (y * 0x40);
                    output.Seek(writeLoc, SeekOrigin.Begin);
                    output.WriteByte((byte)outputChar);
                    output.WriteByte(currPal);

                    writeLoc = loc + (x * 2) + (y * 0x40) + 0x40;
                    output.Seek(writeLoc, SeekOrigin.Begin);
                    output.WriteByte((byte)(outputChar + 0x60));
                    output.WriteByte(currPal);

                    x++;
                }
                
            }

            output.Close();
        }

        //=================================================================================================

        static string FormatMainTextLine(string str, TableReader table, bool squishNewlines = true, bool doAutoNewlines = true)
        {
            string tempStr = str;
            int currLineNum = 0;
            int currCharCount = 0;
            int maxLength = 24;

            str = str.Replace("_", "][");
            doAutoNewlines = false;
            
            int lastSpaceIndex = 0;
            List<byte> convertedData = new List<byte>();
            string[] splitter = { "zlkj8234a" };
            string[] pieces = str.Split(splitter, StringSplitOptions.None);
            
            pieces = str.Split(splitter, StringSplitOptions.None);
            for (int i = 0; i < pieces.Length; i++)
            {
                byte[] tempBytes = eTable.StringToHex(pieces[i], true);
                int iterations = 0;
                
                if (tempBytes != null)
                {
                    iterations = tempBytes.Length - 1;
                }
                else
                {
                    iterations = 0;
                }

                currCharCount = 0;
                for (int j = 0; j < iterations; j++) // now we pick through the data and calculate length junk
                {
                    string ch = tempBytes[j].ToString("X2");

                    if (ch == "02")
                    {
                        lastSpaceIndex = convertedData.Count();
                    }


                    if (ch == "00") // end of line
                    {
                        convertedData.Add(tempBytes[j]);
                        currLineNum++;
                        currCharCount = 0;
                    }
                    else if (ch == "01") // newline
                    {
                        convertedData.Add(0x01);
                    }
                    else if (ch == "02") // tabulation, has 1 byte param
                    {
                        convertedData.Add(tempBytes[j]);
                        j++;
                        convertedData.Add(tempBytes[j]);
                        
                        currCharCount += tempBytes[j];
                    }
                    else if (ch == "03") // change music, has 1 byte param
                    {
                        convertedData.Add(tempBytes[j]);
                        j++;
                        convertedData.Add(tempBytes[j]);
                        //j++;
                    }
                    else if (ch == "04") // print char name, has 1 byte param
                    {
                        if (doAutoNewlines && currCharCount > maxLength - 6)
                        {
                            convertedData.Add(0x01);
                            currCharCount = 0;
                            currLineNum++;
                        }
                        currCharCount += 5;

                        convertedData.Add(tempBytes[j]);
                        j++;
                        convertedData.Add(tempBytes[j]);
                    }
                    else if (ch == "FC") // temp code, these will change into EOL 0x00 codes
                    {
                        currCharCount = 0;
                        currLineNum = 0;
                        convertedData.Add(0xFC);
                    }
                    else if (ch == "05") // pause
                    {
                        convertedData.Add(tempBytes[j]);
                    }
                    else if (ch == "06") // stats or something, might have more bytes to be read
                    {
                        convertedData.Add(tempBytes[j]);
                        currCharCount++;
                    }
                    else if (ch == "07") // item from chest
                    {
                        if (doAutoNewlines)
                        {
                            if (currCharCount > maxLength - 10)  // make sure it prints on a newline if needed
                            {
                                convertedData.Add(0x01);
                                currCharCount = 0;
                                currLineNum++;
                            }

                            convertedData.Add(tempBytes[j]);
                            currCharCount += 9;
                        }
                        else
                        {
                            convertedData.Add(tempBytes[j]);
                        }
                    }
                    else if (ch == "08") // gil
                    {
                        if (doAutoNewlines)
                        {
                            if (currCharCount > maxLength - 6)
                            {
                                convertedData.Add(0x01);
                                currCharCount = 0;
                                currLineNum++;
                            }

                            convertedData.Add(tempBytes[j]);
                            currCharCount += 5;
                        }
                        else
                        {
                            convertedData.Add(tempBytes[j]);
                        }
                    }
                    else if (ch == "09" || ch == "0A" || ch == "0B" || ch == "0C" || ch == "0D" || ch == "0E" ) // unknown
                    {
                        convertedData.Add(tempBytes[j]);
                    }
                    else if (ch == "FF")
                    {
                        convertedData.Add(0xFF);
                    }
                    else
                    {
                        convertedData.Add(tempBytes[j]);
                        currCharCount++;
                    }
                }

                if (i < pieces.Length - 1)
                {
                    if (currCharCount != maxLength)
                    {
                        convertedData.Add(0x01);
                    }
                }
            }

            if (convertedData.Count() > 0 && (byte)convertedData.Last() != 0xFC)
            {
                convertedData.Add(0x00);
            }
            
            string finalString = "";
            foreach (byte b in convertedData)
            {
                if (b == 0xFC)
                {
                    finalString += "[00]";
                }
                else
                {
                    finalString += "[" + b.ToString("X2") + "]";
                }
            }
            str = finalString;

            return str;
        }

        //=================================================================================================

        static int SNESLoROMtoHex(int addr)
        {
            if (addr < 0 || addr > 0xFFFFFF ||    // not 24-bit
                (addr & 0xFE0000) == 0x7E0000 ||  // wram
                (addr & 0x408000) == 0x000000)    // hardware regs
            {
                return -1;
            }
             
            addr = ((addr & 0x7F0000) >> 1 | (addr & 0x7FFF));   
            return addr;
        }
    }

    //========================================================================================/
    //========================================================================================/

    public class TableReader
    {
        Hashtable htoeEntries;
        Hashtable etohEntries;
        string tableFilename;

        public TableReader()
        {
        }

        public TableReader(string filename)
        {
            LoadTextTable(filename);
        }

        //=================================================================================================

        public void LoadTextTable(string filename)
        {
            tableFilename = filename;
            string[] tableLines = System.IO.File.ReadAllLines(tableFilename);
            string[] pieces;

            htoeEntries = new Hashtable();
            etohEntries = new Hashtable();
            foreach (string str in tableLines)
            {
                pieces = str.Split(new Char[] { '=' });

                htoeEntries[pieces[0]] = pieces[1];
                etohEntries[pieces[1]] = pieces[0];
            }
        }

        //=================================================================================================

        public string HexToEntry(int hex)
        {
            string temp = String.Format("{0:X}", hex);
            if (hex < 0x10)
            {
                temp = hex.ToString("X2");
            }

            // this is a custom thing that is only meant for Funky Fantasy IV!
            if (hex == 0x7A)
            {
                return "'";
            }

            if (htoeEntries[temp] != null)
                return (string)htoeEntries[temp];
            else
                return null;
        }

        //=================================================================================================

        public int EntryToHex(string entry)
        {
            string temp = (string)etohEntries[entry];
            if (etohEntries[entry] != null)
            {
                return Int32.Parse((string)etohEntries[entry], System.Globalization.NumberStyles.HexNumber);
            }
            else
            {
                return 0x7F; // return a multiplication sign if the input doesn't exist in the table
            }
        }

        //=================================================================================================

        public int WriteLine(string str, FileStream romFile, long loc, int maxLength = -1, byte fillerByte = 0xFF)
        {
            byte[] bytes = StringToHex(str, true);

            if (bytes == null)
            {
                bytes = new byte[] { fillerByte };
            }
            
            if (maxLength != -1 && bytes.Length > 1 && bytes[bytes.Length - 1] == 0x00)
            {
                bytes[bytes.Length - 1] = fillerByte;
            }

            int bytesToWrite = maxLength;
            if (bytesToWrite == -1 || maxLength > bytes.Count())
            {
                bytesToWrite = bytes.Count();
            }

            int i = 0;
            romFile.Seek(loc, SeekOrigin.Begin);
            for (i = 0; i < bytesToWrite; i++)
            {
                romFile.WriteByte(bytes[i]);
            }

            int bytesLeftToWrite = 0;
            if (maxLength != -1)
            {
                bytesLeftToWrite = maxLength - i;
                for (int j = 0; j < bytesLeftToWrite; j++)
                {
                    romFile.WriteByte(fillerByte);
                }
            }

            return (bytesToWrite + bytesLeftToWrite);
        }

        //=================================================================================================

        public byte[] StringToHex(string str, bool parseBrackets = false)
        {
            ArrayList buffer = new ArrayList();
            byte[] retVal = null;
            int j = 0;
            int i = 0;
            int bracketedByte = -1;

            while (i < str.Length)
            {
                bracketedByte = -1;

                if (parseBrackets)
                {
                    if (str[i] == '[' && (i + 1 < str.Length))
                    {
                        int offset = str.IndexOf("]", i);

                        if (offset != -1)
                        {
                            string subStr = str.Substring(i + 1, offset - i - 1).Trim();

                            try
                            {
                                bracketedByte = Convert.ToInt32(subStr, 16);
                            }
                            catch (Exception e)
                            {
                                bracketedByte = 0xC3; // this is a square character in FFIV, remove this for other projects!!!
                            }

                            buffer.Add((byte)bracketedByte);
                            i = offset;
                        }
                    }
                }

                if (bracketedByte == -1)
                {
                    j = EntryToHex(str[i].ToString());
                    if (j != -1)
                        buffer.Add((byte)j);
                }

                i++;
            }

            if (buffer.Count > 0) 
            {
                if ((byte)buffer[buffer.Count - 1] == 0x00) // we were getting double EOL 0x00s before, so let's check for a 0x00 first
                {
                    byte temp = 0;
                    retVal = new byte[buffer.Count];
                    for (i = 0; i < buffer.Count; i++)
                    {
                        temp = (byte)buffer[i];
                        retVal[i] = temp;
                    }
                }
                else // if there isn't an end 0x00 then let's add one
                {
                    byte temp = 0;
                    retVal = new byte[buffer.Count + 1];
                    for (i = 0; i < buffer.Count; i++)
                    {
                        temp = (byte)buffer[i];
                        retVal[i] = temp;
                    }

                    retVal[i] = 0x00;
                }
            }

            return retVal;
        }



    }
}
